﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace DataTree_Button_MVVM
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
    }

    public class CommandHandler : ICommand
    {
        private Action _action;
        private bool _canExecute;
        public CommandHandler(Action action, bool canExecute)
        {
            _action = action;
            _canExecute = canExecute;
        }

        public bool CanExecute(object parameter)
        {
            return _canExecute;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            _action();
        }
    }




    public class WViewModel : INotifyPropertyChanged
    {

        ObservableCollection<Worker> listOfWorkers;

        public WViewModel()
        {
            listOfWorkers = new ObservableCollection<Worker>();

            listOfWorkers.Add(new Worker("Vivian", 22, "QA"));
            listOfWorkers.Add(new Worker("Andy", 23, "DS"));
            listOfWorkers.Add(new Worker("Peter", 24, "QA"));
            _canExecute = true;

        }

        private ICommand _showReportCommand;
        public ICommand ShowReportCommand
        {
            get
            {
                return _showReportCommand ?? (_showReportCommand = new CommandHandler(() => MyAction(), _canExecute));
            }
        }
        private bool _canExecute;
        public void MyAction()
        {
            MessageBox.Show("ShowReportCommand is executed");
        }

        public ObservableCollection<Worker> ListOfWorkers
        {
            get { return listOfWorkers; }

            set
            {
                if (value != listOfWorkers)
                {
                    listOfWorkers = value;
                    OnPropertyChanged("ListOfWorkers");
                }
            }
        }


        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        public event PropertyChangedEventHandler PropertyChanged;

    }


    public class Worker : INotifyPropertyChanged
    {
        public Worker() { }

        public Worker(string name, int age, string job)
        {
            this.Name = name;
            this.Age = age;
            this.JobTitle = job;
        }


        private string name;

        public string Name
        {
            get
            {
                return this.name;
            }

            set
            {
                if (value != this.name)
                {
                    this.name = value;
                    NotifyPropertyChanged("Name");
                }
            }
        }

        private int age;

        public int Age
        {
            get
            {
                return this.age;
            }

            set
            {
                if (value != this.age)
                {
                    this.age = value;
                    NotifyPropertyChanged("Age");
                }
            }
        }


        private string jobTitle;

        public string JobTitle
        {
            get
            {
                return this.jobTitle;
            }

            set
            {
                if (value != this.jobTitle)
                {
                    this.jobTitle = value;
                    NotifyPropertyChanged("JobTitle");
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }


    }
}
